@@ -27,7 +27,6 @@
 #ifndef DBUS_ARCH_DEPS_H
 #define DBUS_ARCH_DEPS_H
 
-#include <dbus/dbus-types.h>
 #include <dbus/dbus-macros.h>
 
 DBUS_BEGIN_DECLS;
@@ -46,6 +45,12 @@ typedef unsigned long long dbus_uint64_t;
 #undef DBUS_UINT64_CONSTANT
 #endif
 
+typedef int dbus_int32_t;
+typedef unsigned int dbus_uint32_t;
+
+typedef short dbus_int16_t;
+typedef unsigned short dbus_uint16_t;
+
 DBUS_END_DECLS;
 
 #endif /* DBUS_ARCH_DEPS_H */
