@@ -1,10 +1,9 @@
 DBUS_GERROR_FAILED,
 DBUS_GERROR_NO_MEMORY,
-DBUS_GERROR_ACTIVATE_SERVICE_NOT_FOUND,
-DBUS_GERROR_SERVICE_DOES_NOT_EXIST,
-DBUS_GERROR_SERVICE_HAS_NO_OWNER,
+DBUS_GERROR_SERVICE_UNKNOWN,
+DBUS_GERROR_NAME_HAS_NO_OWNER,
 DBUS_GERROR_NO_REPLY,
-DBUS_GERROR_IO_GERROR,
+DBUS_GERROR_IO_ERROR,
 DBUS_GERROR_BAD_ADDRESS,
 DBUS_GERROR_NOT_SUPPORTED,
 DBUS_GERROR_LIMITS_EXCEEDED,
@@ -27,6 +26,9 @@ DBUS_GERROR_SPAWN_CHILD_EXITED,
 DBUS_GERROR_SPAWN_CHILD_SIGNALED,
 DBUS_GERROR_SPAWN_FAILED,
 DBUS_GERROR_UNIX_PROCESS_ID_UNKNOWN,
+DBUS_GERROR_INVALID_SIGNATURE,
+DBUS_GERROR_SELINUX_SECURITY_CONTEXT_UNKNOWN,
+DBUS_GERROR_REMOTE_EXCEPTION
 #ifndef DBUS_INSIDE_DBUS_GLIB_H
 #error " dbus-glib-error-enum.h may only be included by dbus-glib.h"
 #endif
