@@ -76,7 +76,7 @@ static gboolean message_queue_dispatch (GSource     *source,
                                         GSourceFunc  callback,
                                         gpointer     user_data);
 
-static GSourceFuncs message_queue_funcs = {
+static const GSourceFuncs message_queue_funcs = {
   message_queue_prepare,
   message_queue_check,
   message_queue_dispatch,
@@ -286,6 +286,7 @@ connection_setup_add_watch (ConnectionSetup *cs,
   cs->ios = g_slist_prepend (cs->ios, handler);
   
   dbus_watch_set_data (watch, handler, io_handler_watch_freed);
+  g_io_channel_unref (channel);
 }
 
 static void
