@@ -635,13 +635,11 @@ typedef struct
 static void
 unassociate_proxies (gpointer key, gpointer val, gpointer user_data)
 {
-  const char *tri;
   DBusGProxyList *list;
   const char *name;
   GSList *tmp;
   DBusGProxyUnassociateData *data;
 
-  tri = key;
   list = val;
   data = user_data;
   name = data->name;
@@ -2322,7 +2320,8 @@ dbus_g_proxy_end_call (DBusGProxy          *proxy,
  * values.  This function is equivalent to dbus_g_proxy_begin_call
  * followed by dbus_g_proxy_end_call.  All of the input arguments are
  * specified first, followed by G_TYPE_INVALID, followed by all of the
- * output values, followed by G_TYPE_INVALID.
+ * output values, followed by a second G_TYPE_INVALID.  Note that  
+ * this means you must always specify G_TYPE_INVALID twice.
  *
  * @param proxy a proxy for a remote interface
  * @param method method to invoke
