@@ -388,8 +388,8 @@ _dbus_gtype_to_signature (GType gtype)
   return ret;
 }
 
-static char *
-dbus_gvalue_to_signature (const GValue *val)
+char *
+_dbus_gvalue_to_signature (const GValue *val)
 {
   GType gtype;
 
@@ -402,14 +402,16 @@ dbus_gvalue_to_signature (const GValue *val)
 
       array = g_value_get_boxed (val);
       
-      str = g_string_new ("");
+      str = g_string_new (DBUS_STRUCT_BEGIN_CHAR_AS_STRING);
       for (i = 0; i < array->n_values; i++)
 	{
 	  char *sig;
-	  sig = dbus_gvalue_to_signature (g_value_array_get_nth (array, i));
+	  sig = _dbus_gvalue_to_signature (g_value_array_get_nth (array, i));
 	  g_string_append (str, sig);
 	  g_free (sig);
 	}
+      g_string_append (str, DBUS_STRUCT_END_CHAR_AS_STRING);
+      
       return g_string_free (str, FALSE);
     }
   else
@@ -1463,7 +1465,7 @@ marshal_variant (DBusMessageIter          *iter,
   real_value = g_value_get_boxed (value);
   value_gtype = G_VALUE_TYPE (real_value);
 
-  variant_sig = dbus_gvalue_to_signature (real_value);
+  variant_sig = _dbus_gvalue_to_signature (real_value);
   if (variant_sig == NULL)
     {
       g_warning ("Cannot marshal type \"%s\" in variant", g_type_name (value_gtype));
@@ -1476,7 +1478,7 @@ marshal_variant (DBusMessageIter          *iter,
 					 &subiter))
     goto out;
 
-  if (!marshal_basic (&subiter, real_value))
+  if (!_dbus_gvalue_marshal (&subiter, real_value))
     goto out;
 
   if (!dbus_message_iter_close_container (iter, &subiter))
