@@ -48,7 +48,7 @@
  */
 #define N_CLIENT_THREADS 1
 /* It seems like at least 750000 or so iterations reduces the variability to sane levels */
-#define N_ITERATIONS 750000
+#define N_ITERATIONS 2000
 #define N_PROGRESS_UPDATES 20
 /* Don't make PAYLOAD_SIZE too huge because it gets used as a static buffer size */
 #define PAYLOAD_SIZE 0
@@ -103,17 +103,19 @@ static void
 send_echo_method_call (DBusConnection *connection)
 {
   DBusMessage *message;
+  const char *hello = "Hello World!";
+  dbus_int32_t i32 = 123456;
 
   message = dbus_message_new_method_call (ECHO_SERVICE,
                                           ECHO_PATH,
                                           ECHO_INTERFACE,
                                           ECHO_PING_METHOD);
   dbus_message_append_args (message,
-                            DBUS_TYPE_STRING, "Hello World!",
-                            DBUS_TYPE_INT32, 123456,
+                            DBUS_TYPE_STRING, &hello,
+                            DBUS_TYPE_INT32, &i32,
 #if PAYLOAD_SIZE > 0
                             DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE,
-                            payload, PAYLOAD_SIZE,
+                            &payload, PAYLOAD_SIZE,
 #endif
                             DBUS_TYPE_INVALID);
   
@@ -141,7 +143,7 @@ with_or_without_bus_client_filter (DBusConnection     *connection,
                                    ClientData         *cd)
 {
   if (dbus_message_is_signal (message,
-                              DBUS_INTERFACE_ORG_FREEDESKTOP_LOCAL,
+                              DBUS_INTERFACE_LOCAL,
                               "Disconnected"))
     {
       g_printerr ("Client thread disconnected\n");
@@ -188,7 +190,7 @@ no_bus_thread_func (void *data)
   g_printerr ("Starting client thread %p\n", g_thread_self());  
   
   dbus_error_init (&error);
-  connection = dbus_connection_open (messages_address, &error);
+  connection = dbus_connection_open_private (messages_address, &error);
   if (connection == NULL)
     {
       g_printerr ("could not open connection: %s\n", error.message);
@@ -217,7 +219,7 @@ no_bus_thread_func (void *data)
   g_printerr ("Client thread %p exiting main loop\n",
               g_thread_self());
 
-  dbus_connection_disconnect (connection);
+  dbus_connection_close (connection);
   
   g_main_loop_unref (cd.loop);
   g_main_context_unref (context);
@@ -233,7 +235,7 @@ no_bus_server_filter (DBusConnection     *connection,
   ServerData *sd = user_data;
   
   if (dbus_message_is_signal (message,
-                              DBUS_INTERFACE_ORG_FREEDESKTOP_LOCAL,
+                              DBUS_INTERFACE_LOCAL,
                               "Disconnected"))
     {
       g_printerr ("Client disconnected from server\n");
@@ -277,10 +279,6 @@ no_bus_init_server (ServerData       *sd)
 {
   DBusServer *server;
   DBusError error;
-  
-#ifndef DBUS_DISABLE_ASSERT
-  g_printerr ("You should probably --disable-asserts before you profile as they have noticeable overhead\n");
-#endif
 
   dbus_error_init (&error);
   server = dbus_server_listen ("unix:tmpdir="DBUS_TEST_SOCKET_DIR,
@@ -305,10 +303,9 @@ no_bus_init_server (ServerData       *sd)
 
 static void
 no_bus_stop_server (ServerData *sd,
-                      void       *server)
+                    void       *server)
 {
-  g_printerr ("The following g_warning is because we try to call g_source_remove_poll() after g_source_destroy() in dbus-gmain.c, I think we need to add a source free func that clears out the watch/timeout funcs\n");
-  
+  dbus_server_disconnect (server);
   dbus_server_unref (server);
 }
 
@@ -364,7 +361,7 @@ with_bus_thread_func (void *data)
     }
   
   dbus_error_init (&error);
-  connection = dbus_connection_open (address, &error);
+  connection = dbus_connection_open_private (address, &error);
   if (connection == NULL)
     {
       g_printerr ("could not open connection to bus: %s\n", error.message);
@@ -399,7 +396,7 @@ with_bus_thread_func (void *data)
   g_printerr ("Client thread %p exiting main loop\n",
               g_thread_self());
 
-  dbus_connection_disconnect (connection);
+  dbus_connection_close (connection);
   
   g_main_loop_unref (cd.loop);
   g_main_context_unref (context);
@@ -415,29 +412,29 @@ with_bus_server_filter (DBusConnection     *connection,
   WithBusServer *server = user_data;
   
   if (dbus_message_is_signal (message,
-                              DBUS_INTERFACE_ORG_FREEDESKTOP_LOCAL,
+                              DBUS_INTERFACE_LOCAL,
                               "Disconnected"))
     {
       g_printerr ("Server disconnected from message bus\n");
       exit (1);
     }
   else if (dbus_message_has_sender (message,
-                                    DBUS_SERVICE_ORG_FREEDESKTOP_DBUS) &&
+                                    DBUS_SERVICE_DBUS) &&
            dbus_message_is_signal (message,
-                                   DBUS_INTERFACE_ORG_FREEDESKTOP_DBUS,
-                                   "ServiceOwnerChanged"))
+                                   DBUS_INTERFACE_DBUS,
+                                   "NameOwnerChanged"))
     {
-      char *service_name, *old_owner, *new_owner;
+      const char *name, *old_owner, *new_owner;
       DBusError error;
 
-      service_name = NULL;
+      name = NULL;
       old_owner = NULL;
       new_owner = NULL;
 
       dbus_error_init (&error);
       if (!dbus_message_get_args (message,
                                   &error,
-                                  DBUS_TYPE_STRING, &service_name,
+                                  DBUS_TYPE_STRING, &name,
                                   DBUS_TYPE_STRING, &old_owner,
                                   DBUS_TYPE_STRING, &new_owner,
                                   DBUS_TYPE_INVALID))
@@ -447,20 +444,16 @@ with_bus_server_filter (DBusConnection     *connection,
         }
 
       if (g_hash_table_lookup (server->client_names,
-                               service_name) &&
+                               name) &&
           *old_owner != '\0' &&
           *new_owner == '\0')
         {
           g_hash_table_remove (server->client_names,
-                               service_name);
+                               name);
           server->sd->n_clients -= 1;
           if (server->sd->n_clients == 0)
             g_main_loop_quit (server->sd->loop);
         }
-
-      dbus_free (service_name);
-      dbus_free (old_owner);
-      dbus_free (new_owner);
     }
   else if (dbus_message_is_method_call (message,
                                         ECHO_INTERFACE,
@@ -511,14 +504,6 @@ with_bus_init_server (ServerData       *sd)
       g_printerr ("You have to run with_bus mode with the run-test.sh script\n");
       exit (1);
     }
-  
-#ifndef DBUS_DISABLE_ASSERT
-  g_printerr ("You should probably --disable-asserts before you profile as they have noticeable overhead\n");
-#endif
-  
-#ifdef DBUS_ENABLE_VERBOSE_MODE
-  g_printerr ("You should probably --disable-verbose-mode before you profile as verbose has noticeable overhead\n");
-#endif
 
   /* Note that we use the standard global bus connection for the
    * server, and the clients open their own connections so they can
@@ -539,13 +524,13 @@ with_bus_init_server (ServerData       *sd)
   
   connection = dbus_g_connection_get_connection (gconnection);
 
-  dbus_bus_acquire_service (connection,
-                            ECHO_SERVICE,
-                            0, NULL); /* ignore errors because we suck */
+  dbus_bus_request_name (connection,
+                         ECHO_SERVICE,
+                         0, NULL); /* ignore errors because we suck */
   
   rule = g_strdup_printf ("type='signal',sender='%s',member='%s'",
-                          DBUS_SERVICE_ORG_FREEDESKTOP_DBUS,
-                          "ServiceOwnerChanged");
+                          DBUS_SERVICE_DBUS,
+                          "NameOwnerChanged");
 
   /* ignore errors because we suck */
   dbus_bus_add_match (connection, rule, NULL);
@@ -669,7 +654,7 @@ write_junk (int fd,
   int val;
   char *buf;
   char *allocated;
-  char not_allocated[512+PAYLOAD_SIZE];
+  char not_allocated[512+PAYLOAD_SIZE] = { '\0', };
 
   g_assert (count < (int) sizeof(not_allocated));
   
@@ -993,6 +978,7 @@ plain_sockets_thread_func (void *data)
 
   cd.iterations = 1;
   cd.loop = g_main_loop_new (context, FALSE);
+  cd.vtable = data;
 
   channel = g_io_channel_unix_new (fd);
   
@@ -1111,6 +1097,14 @@ main (int argc, char *argv[])
   g_thread_init (NULL);
   dbus_g_thread_init ();
   
+#ifndef DBUS_DISABLE_ASSERT
+  g_printerr ("You should probably --disable-asserts before you profile as they have noticeable overhead\n");
+#endif
+  
+#if DBUS_ENABLE_VERBOSE_MODE
+  g_printerr ("You should probably --disable-verbose-mode before you profile as verbose has noticeable overhead\n");
+#endif
+  
   payload = g_malloc (PAYLOAD_SIZE);
 
   /* The actual size of the DBusMessage on the wire, as of Nov 23 2004,
